// =====================================================================================================================
//
// Main file for the BMC Remedy change integration service.
//
// =====================================================================================================================
importPackage(java.util);
importPackage(java.io);
importPackage(java.text);

importClass(java.lang.Thread);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessageImpl);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.alarmpoint.integrationagent.security.EncryptionUtils);
importClass(Packages.com.alarmpoint.integrationagent.soap.exception.SOAPRequestException);

importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

importClass(Packages.org.apache.commons.httpclient.Header);
importClass(Packages.org.apache.commons.httpclient.HttpVersion);
importClass(Packages.org.mule.providers.http.HttpResponse);

var INTEGRATION_VERSION_NUMBER = "50";

// Core Javascript files provided by the IA
load("integrationservices/lib/javascript/core/baseclass.js");
load("integrationservices/lib/javascript/core/logger.js");
load("integrationservices/lib/javascript/webservices/wsutil.js");
load("integrationservices/lib/javascript/webservices/soapfault.js");
load("integrationservices/lib/javascript/xmatters/xmattersws.js");

// REST API support
load("lib/integrationservices/javascript/log.js");
load("lib/integrationservices/javascript/xmutil.js");
load("lib/integrationservices/javascript/apxml.js");
load("lib/integrationservices/javascript/xmio.js");

// Integration-specific Javascript files
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/util.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/xmrestapi.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/remedyio.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/configuration.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/xm-chg-infrastructure-change-ws.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/xm-chg-change-interface-ws.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/xm-chg-change-ap-detail-signature-ws.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/updateChangeRequest.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/bmcremedychange-events.js");
load("integrationservices/applications/bmcremedychange"+INTEGRATION_VERSION_NUMBER+"/bmcremedychange-callbacks.js");

var xStream = new XStream(new PureJavaReflectionProvider());
var exceptionString = "JavaException: ";


// -----------------------------------------------------------------------------
// Utility functions
// -----------------------------------------------------------------------------

/**
 * getInfrastructureChangeId
 *
 * Check whether additionalTokens and incident_id are present, and return the content if they are.
 * If not, an exception is thrown.
 *
 * @param msg
 */
function getInfrastructureChangeId( msg ) {
  return getProperty( msg, "infrastructure_change_id" );
}

/**
 * getRequestId
 *
 * Check whether additionalTokens and incident_id are present, and return the content if they are.
 * If not, an exception is thrown.
 *
 * @param msg
 */
function getRequestId( msg ) {
  return getProperty( msg, "request_id" );
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * getProperty
 *
 * Check whether additionalTokens and property with propertyName name are present, and return the content if they are.
 * If not, an exception is thrown.
 *
 * @param msg
 * @param propertyName
 * ---------------------------------------------------------------------------------------------------------------------
 */
function getProperty( msg, propertyName, defaultValue ) {
  var hasID = ('additionalTokens' in msg) && (propertyName in msg.additionalTokens);

  if ( !hasID ) {
    if (typeof defaultValue !== "undefined") {
      return defaultValue;
    } else {
      throw { name: "DataException", message: propertyName + " property is not found in event callback data of type " + msg.xmatters_callback_type + " for event ID " + msg.eventidentifier
      + ". Make sure 'Include in Callbacks' is set for " + propertyName + " property in the form's layout"};
    }
  }

  return msg.additionalTokens[propertyName];
}

/**
 * This function was created to log all xMatters notification deliveries.
 * The intent is that the log defined by "com.alarmpoint.notification.delivery" would fulfill the role of an audit log.
 * Enable / Disable logging by settting LOGGING_ENABLED flag in configuration.js
 */
function logNotification(msg) {
  if (LOGGING_ENABLED) {
    log.info(msg);
  }
}

/**
 * The IB response options are Approve, Reject and Hold.
 * This function is to convert them to Remedy Change Approval_Status2 valid ones.
 * @param {string} response: response option
 * @returns {string} the Remedy-valid Approval_Status2 string
 */
function generateApprovalStatus(response) {
  var result = response;

  switch (response.toUpperCase()) {
    case RESPONSE_ACTION_APPROVED:
    case RESPONSE_ACTION_APPROVE:
      result = "Approved";
      break;
    case RESPONSE_ACTION_REJECTED:
    case RESPONSE_ACTION_REJECT:
      result = "Rejected";
      break;

    default:
      break;
  }

  return result;
}

/**
 * New lifecycle method not used in this integration, having this here prevents a nuisance stacktrace
 */
function apia_webservice_init(){}


