XMRESTAPI = {};

(function(){

  XMRESTAPI.RESPONSE_SUCCESS = "200";
  XMRESTAPI.RESPONSE_ACCEPTED = "202";
  XMRESTAPI.RESPONSE_DATA_VALIDATION_ERROR = "400";

  // private functions

  function getEventFilterAsString(filter) {
    var text = "";
    var propertyName = "";
    var propertyValue = "";

    for (var property in filter) {
      var value = filter[property];

      if (property == "properties") {
        // If property-value pair is an array, flatten it.
        var properties = "";
        for (var p in value) {
          propertyName += encodeURIComponent(p) + ",";
          propertyValue += encodeURIComponent(String(value[p])) + ",";
        }
      } else {
        propertyName += encodeURIComponent(property) + ",";
        propertyValue += encodeURIComponent(String(value)) + ",";
      }
    }

    if (propertyName.length > 1) {
      text = "propertyName=" + propertyName.replace(/,\s*$/, "") + "&propertyValue=" + propertyValue.replace(/,\s*$/, "");
    }
    return text;
  }

  // public functions
  XMRESTAPI.getEvents = function(integrationTriggerURL, eventFilter) {
    IALOG.debug("XM REST API: getEvents for " + eventFilter);
    var parameters = getEventFilterAsString(eventFilter);

    var url = integrationTriggerURL.substring(0, integrationTriggerURL.indexOf("api/integration")) + "api/xm/1/events?status=ACTIVE,SUSPENDED&" + parameters;
    IALOG.debug("XM REST API: getEvents url " + url);

    var response = XMIO.get(url);
    IALOG.debug("XM REST API: getEvents received " + response.status + " " + response.body);

    XMRESTAPI.checkResponse( response );
    return JSON.parse(response.body);
  };

  XMRESTAPI.setEventStatus = function(integrationTriggerURL, eventFilter, status) {
    IALOG.debug("XM REST API: setEventStatus for " + eventFilter + " to " + status);
    var count = 0;

    var events = XMRESTAPI.getEvents( integrationTriggerURL, eventFilter );

    for (var i = 0; i < events.count; i++) {
      var event = events.data[i];
      IALOG.debug("XM REST API: post event {0} with {1}", event.id, status);
      var url = integrationTriggerURL.substring(0, integrationTriggerURL.indexOf("api/integration")) + "api/xm/1/events";

      var payload = {
        'id': event.id,
        'status': status
      };
      var response = XMIO.post(JSON.stringify(payload), url);
      IALOG.debug("XM REST API: setEventStatus received {0} {1}", response.status, response.body);
      XMRESTAPI.checkResponse( response, {status : 409} ); // ignore conflict errors

      count++;
    }

    IALOG.debug("XM REST API: setEventStatus events {0}: {1}", status, count);
    return count;
  };

  XMRESTAPI.deleteEvents = function(integrationTriggerURL, eventFilter) {
    IALOG.debug("XM REST API: deleteEvents for {0}", eventFilter);
    return XMRESTAPI.setEventStatus(integrationTriggerURL, eventFilter, "TERMINATED");
  };

  // Submit Apxml
  XMRESTAPI.submitApxml = function(integrationTriggerURL, apxml, existingEventsFilter, keys, deduplicationFilter) {
    log.debug("Enter - submitApxml");
    apxml.setToken("integrationTriggerURL", integrationTriggerURL);

    if (deduplicationFilter !== false && XMUtil.deduplicate(apxml)) {
      return;
    }

    if ( existingEventsFilter ) {
      XMRESTAPI.deleteEvents( integrationTriggerURL, existingEventsFilter );
    }

    var eventObj = XMUtil.createEventTemplate();
    var newKeys = keys ? keys : null;
    var apxmlAsObj = APXML.toEventJs(apxml, eventObj, newKeys);
    var json = JSON.stringify(apxmlAsObj);

    IALOG.debug("XM REST API: Post to {0} {1}", integrationTriggerURL, json);

    log.debug("Exit - submitApxml");
    return XMIO.post(json, integrationTriggerURL);
  }

  // Utility methods
  XMRESTAPI.checkResponse = function(response, ignoreError) {

    if ( response.status !== undefined && response.status != XMRESTAPI.RESPONSE_SUCCESS) {
      // Ignore status?
      if (ignoreError && ignoreError['status'] != null && ignoreError['status'] == response.status) {
        IALOG.debug("XM REST API: checkResponse status {0} will be treated as success", response.status);
        return response;
      }
      var error;
      try {
        var body = JSON.parse(response.body);

        if (ignoreError && ignoreError['type'] != null && ignoreError['type'] == body.type) {
          IALOG.debug("XM REST API: checkResponse error type {0} will be treated as success", response.status);
          return response;
        }
        error = body.message;
      } catch (e) {
        error = "xMatters server returned status " + response.status;
      }
      throw error;
    }

    return response;
  };
})();